#!/usr/bin/env python
import logging
import interval, interval_list,  note

class ChordNote(interval_list.IntervalListNote):
    pass


class ChordType(interval_list.InterValListFormula):
    def __init__(self, name=u'Unknown Chord', 
                 symbol=u'Uknwn', interval_list=[]):
        self.symbol = symbol
        super(ChordType, self).__init__(name=name, interval_list=interval_list)


class Chord(interval_list.IntervalList):
    pass

# Taken from: http://www.smithfowler.org/music/Chord_forumulas.html

TYPES = []

MAJOR = ChordType(u'Major', u'Maj', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.PERFECT_5TH
])
TYPES.append(MAJOR)
ADDED_FOURTH = ChordType(u'Added Fourth', u'add4', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.PERFECT_4TH,
        interval.PERFECT_5TH
])
TYPES.append(ADDED_FOURTH)
SIXTH = ChordType(u'Sixth', u'6', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.PERFECT_5TH,
        interval.MAJOR_6TH,
])
TYPES.append(SIXTH)
SIX_NINE = ChordType(u'Six Nine', u'6/9', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.PERFECT_5TH,
        interval.MAJOR_6TH,
        interval.MAJOR_9TH
])
TYPES.append(SIX_NINE)
MAJOR_SEVENTH = ChordType(u'Major 7th', u'Maj7', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.PERFECT_5TH,
        interval.MAJOR_7TH
])
TYPES.append(MAJOR_SEVENTH)
MAJOR_NINTH = ChordType(u'Major Ninth', 'Maj9', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.PERFECT_5TH,
        interval.MAJOR_7TH,
        interval.MAJOR_9TH
])
TYPES.append(MAJOR_NINTH)
MAJOR_ELEVENTH = ChordType(u'Major Eleventh', 'Maj11', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.PERFECT_5TH,
        interval.MAJOR_7TH,
        interval.MAJOR_9TH,
        interval.MAJOR_11TH
])
TYPES.append(MAJOR_ELEVENTH)
MAJOR_THIRTEENTH = ChordType(u'Major Thirteenth', 'Maj13', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.PERFECT_5TH,
        interval.MAJOR_7TH,
        interval.MAJOR_9TH,
        interval.MAJOR_11TH,
        interval.MAJOR_13TH
])
TYPES.append(MAJOR_THIRTEENTH)
MAJOR_SEVEN_SHARP_ELEVENTH = ChordType(u'Major seven sharp eleventh', 'Maj7#11', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.PERFECT_5TH,
        interval.MAJOR_7TH,
        interval.SHARP_11TH,
])
TYPES.append(MAJOR_SEVEN_SHARP_ELEVENTH)
MAJOR_FLAT_FIVE = ChordType(u'Major Flat Five', 'Maj b5', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.DIMINISHED_5TH
])
TYPES.append(MAJOR_FLAT_FIVE)

MINOR = ChordType(u'Minor', 'm', [
        interval.PERFECT_UNISON,
        interval.MINOR_3RD,
        interval.PERFECT_5TH
])
TYPES.append(MINOR)
MINOR_ADDED_FOURTH = ChordType(u'Minor added fourth', 'madd4', [
        interval.PERFECT_UNISON,
        interval.MINOR_3RD,
        interval.PERFECT_4TH,
        interval.PERFECT_5TH
])
TYPES.append(MINOR_ADDED_FOURTH)
MINOR_SIXTH = ChordType(u'Minor sixth', 'm6', [
        interval.PERFECT_UNISON,
        interval.MINOR_3RD,
        interval.PERFECT_5TH,
        interval.MAJOR_6TH
])
TYPES.append(MINOR_SIXTH)
MINOR_SEVENTH = ChordType(u'Minor seventh', 'm7', [
        interval.PERFECT_UNISON,
        interval.MINOR_3RD,
        interval.PERFECT_5TH,
        interval.MINOR_7TH
])
TYPES.append(MINOR_SEVENTH)
MINOR_ADDED_NINTH = ChordType(u'Minor added ninth', 'madd9', [
        interval.PERFECT_UNISON,
        interval.MINOR_3RD,
        interval.PERFECT_5TH,
        interval.MAJOR_9TH
])
TYPES.append(MINOR_ADDED_NINTH)
MINOR_SIX_ADD_NINE = ChordType(u'Minor six add nine', 'm6/9', [
        interval.PERFECT_UNISON,
        interval.MINOR_3RD,
        interval.PERFECT_5TH,
        interval.MAJOR_6TH,
        interval.MAJOR_9TH
])
TYPES.append(MINOR_SIX_ADD_NINE)
MINOR_NINTH = ChordType(u'Minor ninth', 'm9', [
        interval.PERFECT_UNISON,
        interval.MINOR_3RD,
        interval.PERFECT_5TH,
        interval.MINOR_7TH,
        interval.MAJOR_9TH
])
TYPES.append(MINOR_NINTH)
MINOR_ELEVENTH = ChordType(u'Minor eleventh', 'm11', [
        interval.PERFECT_UNISON,
        interval.MINOR_3RD,
        interval.PERFECT_5TH,
        interval.MINOR_7TH,
        interval.MAJOR_9TH,
        interval.MAJOR_11TH
])
TYPES.append(MINOR_ELEVENTH)
MINOR_THIRTEENTH = ChordType(u'Minor thirteenth', 'm13', [
        interval.PERFECT_UNISON,
        interval.MINOR_3RD,
        interval.PERFECT_5TH,
        interval.MINOR_7TH,
        interval.MAJOR_9TH,
        interval.MAJOR_11TH,
        interval.MAJOR_13TH
])
TYPES.append(MINOR_THIRTEENTH)
MINOR_MAJOR_SEVENTH = ChordType(u'Minor/Major seventh', 'm/Maj7', [
        interval.PERFECT_UNISON,
        interval.MINOR_3RD,
        interval.PERFECT_5TH,
        interval.MAJOR_7TH
])
TYPES.append(MINOR_MAJOR_SEVENTH)
MINOR_MAJOR_NINTH = ChordType(u'Minor/Major ninth', 'm/Maj9', [
        interval.PERFECT_UNISON,
        interval.MINOR_3RD,
        interval.PERFECT_5TH,
        interval.MAJOR_7TH,
        interval.MAJOR_9TH
])
TYPES.append(MINOR_MAJOR_NINTH)
MINOR_MAJOR_ELEVENTH = ChordType(u'Minor/Major eleventh', 'm/Maj11', [
        interval.PERFECT_UNISON,
        interval.MINOR_3RD,
        interval.PERFECT_5TH,
        interval.MAJOR_7TH,
        interval.MAJOR_9TH,
        interval.MAJOR_11TH
])
TYPES.append(MINOR_MAJOR_ELEVENTH)
MINOR_MAJOR_THIRTEENTH = ChordType(u'Minor/Major thirteenth', 'm/Maj13', [
        interval.PERFECT_UNISON,
        interval.MINOR_3RD,
        interval.PERFECT_5TH,
        interval.MAJOR_7TH,
        interval.MAJOR_9TH,
        interval.MAJOR_11TH,
        interval.MAJOR_13TH
])
TYPES.append(MINOR_MAJOR_THIRTEENTH)
MINOR_SEVEN_FLAT_FIFTH = ChordType(u'Minor seven flat fifth', 'm7-5', [
        interval.PERFECT_UNISON,
        interval.MINOR_3RD,
        interval.TRITONE,
        interval.MINOR_7TH,
])
TYPES.append(MINOR_SEVEN_FLAT_FIFTH)

DOMINANT_SEVENTH = ChordType(u'Dominant seventh', '7', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.PERFECT_5TH,
        interval.MINOR_7TH,
])
TYPES.append(DOMINANT_SEVENTH)
DOMINANT_NINTH = ChordType(u'Dominant ninth', '9', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.PERFECT_5TH,
        interval.MINOR_7TH,
        interval.MAJOR_9TH,
])
TYPES.append(DOMINANT_NINTH)
DOMINANT_ELEVENTH = ChordType(u'Dominant eleventh', '11', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.PERFECT_5TH,
        interval.MINOR_7TH,
        interval.MAJOR_9TH,
        interval.MAJOR_11TH,
])
TYPES.append(DOMINANT_ELEVENTH)
DOMINANT_THIRTEENTH = ChordType(u'Dominant thirteenth', '13', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.PERFECT_5TH,
        interval.MINOR_7TH,
        interval.MAJOR_9TH,
        interval.MAJOR_11TH,
        interval.MAJOR_13TH,
])
TYPES.append(DOMINANT_THIRTEENTH)
DOMINANT_SEVEN_SHARP_FIVE = ChordType(u'Dominant seven sharp five', '7#5', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.SHARP_5TH,
        interval.MINOR_7TH,
])
TYPES.append(DOMINANT_SEVEN_SHARP_FIVE)
DOMINANT_SEVEN_FLAT_FIVE = ChordType(u'Dominant seven flat five', '7b5', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.FLAT_5TH,
        interval.MINOR_7TH,
])
TYPES.append(DOMINANT_SEVEN_FLAT_FIVE)
DOMINANT_SEVEN_FLAT_NINTH = ChordType(u'Dominant seven flat ninth', '7b9', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.PERFECT_5TH,
        interval.MINOR_7TH,
        interval.MINOR_9TH,
])
TYPES.append(DOMINANT_SEVEN_FLAT_NINTH)
DOMINANT_SEVEN_SHARP_NINTH = ChordType(u'Dominant seven sharp ninth', '7#9', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.PERFECT_5TH,
        interval.MINOR_7TH,
        interval.SHARP_9TH,
])
TYPES.append(DOMINANT_SEVEN_SHARP_NINTH)
DOMINANT_NINE_SHARP_FIVE = ChordType(u'Dominant nine sharp five', '9#5', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.SHARP_5TH,
        interval.MINOR_7TH,
        interval.MAJOR_9TH,
])
TYPES.append(DOMINANT_NINE_SHARP_FIVE)
DOMINANT_NINE_FLAT_FIVE = ChordType(u'Dominant nine flat five', '9b5', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.FLAT_5TH,
        interval.MINOR_7TH,
        interval.MAJOR_9TH,
])
TYPES.append(DOMINANT_NINE_FLAT_FIVE)
DOMINANT_SEVEN_SHARP_FIVE_SHARP_NINE = ChordType(u'Dominant seventh sharp five sharp nine', '7#5#9', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.SHARP_5TH,
        interval.MINOR_7TH,
        interval.SHARP_9TH,
])
TYPES.append(DOMINANT_SEVEN_SHARP_FIVE_SHARP_NINE)
DOMINANT_SEVEN_SHARP_FIVE_FLAT_NINE = ChordType(u'Dominant seventh sharp five flat nine', '7#5b9', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.SHARP_5TH,
        interval.MINOR_7TH,
        interval.MINOR_9TH,
])
TYPES.append(DOMINANT_SEVEN_SHARP_FIVE_FLAT_NINE)
DOMINANT_SEVEN_FLAT_FIVE_SHARP_NINE = ChordType(u'Dominant seventh flat five sharp nine', '7b5#9', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.FLAT_5TH,
        interval.MINOR_7TH,
        interval.SHARP_9TH,
])
TYPES.append(DOMINANT_SEVEN_FLAT_FIVE_SHARP_NINE)
DOMINANT_SEVEN_FLAT_FIVE_FLAT_NINE = ChordType(u'Dominant seventh flat five flat nine', '7b5b9', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.FLAT_5TH,
        interval.MINOR_7TH,
        interval.MINOR_9TH,
])
TYPES.append(DOMINANT_SEVEN_FLAT_FIVE_FLAT_NINE)
DOMINANT_SEVEN_SHARP_ELEVEN = ChordType(u'Dominant seventh sharp eleven', '7#11', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.PERFECT_5TH,
        interval.MINOR_7TH,
        interval.SHARP_11TH,
])
TYPES.append(DOMINANT_SEVEN_SHARP_ELEVEN)

DIMINISHED = ChordType(u'Diminished', 'dim', [
        interval.PERFECT_UNISON,
        interval.MINOR_3RD,
        interval.FLAT_5TH,
])
TYPES.append(DIMINISHED)
DIMINISHED_SEVENTH = ChordType(u'Diminished seventh', 'dim7', [
        interval.PERFECT_UNISON,
        interval.MINOR_3RD,
        interval.FLAT_5TH,
        interval.DIMINISHED_7TH,
])
TYPES.append(DIMINISHED_SEVENTH)
AUGMENTED = ChordType(u'Augmented', 'aug', [
        interval.PERFECT_UNISON,
        interval.MAJOR_3RD,
        interval.FLAT_5TH,
])
TYPES.append(AUGMENTED)

FIFTH = ChordType(u'Fifth', '5', [
        interval.PERFECT_UNISON,
        interval.PERFECT_5TH,
])
TYPES.append(FIFTH)
FLAT_FIFTH = ChordType(u'Flat fifth', '-5', [
        interval.PERFECT_UNISON,
        interval.FLAT_5TH,
])
TYPES.append(FLAT_FIFTH)
SUSPENDED_FOURTH = ChordType(u'Suspended fourth', 'sus4', [
        interval.PERFECT_UNISON,
        interval.PERFECT_4TH,
        interval.PERFECT_5TH,
])
TYPES.append(SUSPENDED_FOURTH)
SUSPENDED_SECOND = ChordType(u'Suspended second', 'sus2', [
        interval.PERFECT_UNISON,
        interval.MAJOR_2ND,
        interval.MAJOR_9TH,
        interval.PERFECT_5TH,
])
TYPES.append(SUSPENDED_SECOND)
SHARP_ELEVEN = ChordType(u'Sharp eleven', '#11', [
        interval.PERFECT_UNISON,
        interval.PERFECT_5TH,
        interval.SHARP_11TH,
])
TYPES.append(SHARP_ELEVEN)











TYPES_BY_NAME = {
}

def normalize_chord_name(name):
    name = name.strip()
    name = name.lower()
    name = name.replace(' ', '_')
    name = name.replace('/', '_')
    name = name.replace('-', '_')
    while '__' in name:
        name = name.replace('__', '_')
    return name

for type in TYPES:
    TYPES_BY_NAME[normalize_chord_name(type.name)] = type


def get_all_chords():
    chord_list = []
    for root in note.NOTES:
        for type in TYPES:
            chord_list.append(Chord(type, root))
    return chord_list

