import logging

try:
    import readline
except ImportError:
    pass
import cmd

class MusicTheoryCmd(cmd.Cmd):
    def __init__(self, command=None, *args, **kwargs):
        from fretboard import STANDARD_GUITAR
        cmd.Cmd.__init__(self, *args, **kwargs)
        self.prompt = u'pymt> '
        self.doc_header = u''
        self.misc_header = u''
        self.undoc_header = u''
        self.should_run = True
        self.cur_fretboard = STANDARD_GUITAR
        if command:
            self.onecmd(command)
        else:
            self.go_interactive()

    def go_interactive(self):
        while self.should_run:
            self.cmdloop()

    def do_describe_scale(self, line):
        scale = self.parse_scale(line)
        print scale.description

    def do_describe_chord(self, line): 
        chord = self.parse_chord(line)
        print chord.description

    def do_clear_fretboard(self, line):
        self.cur_fretboard.clear()

    def do_fret_scale(self, line):
        self.onecmd('clear_fretboard')
        scale = self.parse_scale(line)
        self.cur_fretboard.display_scale(scale)
        print self.cur_fretboard.ascii_diagram

    def do_fret_chord(self, line):
        self.onecmd('clear_fretboard')
        chord = self.parse_chord(line)
        self.cur_fretboard.display_chord(chord)
        print self.cur_fretboard.ascii_diagram

    def complete_describe_scale(self, *args):
        return self.complete_scale(*args)

    def complete_describe_chord(self, *args):
        return self.complete_chord(*args)

    def complete_fret_scale(self, *args):
        return self.complete_scale(*args)

    def complete_fret_chord(self, *args):
        return self.complete_chord(*args)

    def do_set_num_frets(self, line):
        try:
            line = line.strip()
            self.cur_fretboard.num_frets = int(line)
        except:
            print 'Error setting num_frets'

    def parse_scale(self, line):
        from scale import TYPES_BY_NAME, CHROMATIC, Scale, normalize_scale_name
        from note import NOTES_BY_NAME, C
        try:
            note_name, scale_name = line.split(' ')
        except ValueError, e:
            scale_name = line
            note_name = 'c'
        scale_name = normalize_scale_name(scale_name)
        note_name = note_name.lower()
        root_note = NOTES_BY_NAME.get(note_name, C)
        scale_type = TYPES_BY_NAME.get(scale_name, CHROMATIC)
        return Scale(scale_type, root_note)

    def complete_scale(self, text, line, begidx, endidx):
        from scale import TYPES_BY_NAME
        match_list = []
        for name in TYPES_BY_NAME.keys():
            if name.startswith(text):
                match_list.append(name)
        match_list.sort()
        return match_list

    def parse_chord(self, line):
        from chord import TYPES_BY_NAME, MAJOR, Chord, normalize_chord_name
        from note import NOTES_BY_NAME, C
        try:
            note_name, chord_name = line.split(' ')
        except ValueError, e:
            chord_name = line
            note_name = 'c'
        chord_name = normalize_chord_name(chord_name)
        note_name = note_name.lower()
        root_note = NOTES_BY_NAME.get(note_name, C)
        chord_type = TYPES_BY_NAME.get(chord_name, MAJOR)
        return Chord(chord_type, root_note)

    def complete_chord(self, text, line, begidx, endidx):
        from chord import TYPES_BY_NAME
        match_list = []
        for name in TYPES_BY_NAME.keys():
            if name.startswith(text):
                match_list.append(name)
        match_list.sort()
        return match_list

    def do_quit(self, line):
        self.should_run = False

    def postcmd(self, stop, line):
        if not self.should_run:
            return 1
        return stop

if __name__=='__main__':
    app = MusicTheoryCmd()

