class Interval(object):
    def __init__(self, number=1, semitone_adjustment=0):
        self.number = number 
        self.semitone_adjustment = semitone_adjustment

    def __hash__(self):
        return self.number + self.semitone_adjustment

    def __cmp__(self, right):
        if hasattr(right, 'num_semitones'):
            return cmp(self.num_semitones, right.num_semitones)
        return cmp(id(self), id(right))

    def get_num_semitones(self):
        pattern = [0, 2, 2, 1, 2, 2, 2, 1]
        num_semitones = 0
        for step in range(self.number):
            num_semitones += pattern[step % len(pattern)]
        num_semitones += self.semitone_adjustment
        return num_semitones
    num_semitones = property(get_num_semitones)

    def get_size_name(self):
        if self.number ==1:
            return u'Unison'
        elif self.number == 2:
            return u'2nd'
        elif self.number == 3:
            return u'3rd'
        elif self.number == 8:
            return u'Octave'
        else:
            return u'%ith' % self.number
    size_name = property(get_size_name)

    def get_quality_name(self):
        if self.is_major:
            return 'Major'
        elif self.is_minor:
            return 'Minor'
        elif self.is_perfect:
            return 'Perfect'
        elif self.is_augmented:
            return 'Augmented'
        elif self.is_diminished:
            return 'Diminished'
    quality_name = property(get_quality_name)

    def get_is_major(self):
        # 2nd, 3rd, 6th and 7th are major intervals
        if self.number in (2, 3, 6, 7) and self.semitone_adjustment == 0:
            return True
        else:
            return False
    is_major = property(get_is_major)

    def get_is_minor(self):
        # If you flat a major interval it becomes minor
        if self.number in (2, 3, 6, 7) and self.semitone_adjustment == -1:
            return True
        else:
            return False
    is_minor = property(get_is_minor)

    def get_is_perfect(self):
        # Unison, 4th, 5th and octave are perfect intervals
        if self.number in (1, 4, 5, 8) and self.semitone_adjustment == 0:
            return True
        else:
            return False
    is_perfect = property(get_is_perfect)

    def get_is_augmented(self):
        # If you sharp a perfect interval it becomes augmented
        # If you sharp a major interval it becomes augmented
        if self.number in (1, 4, 5, 8) and self.semitone_adjustment == 1:
            return True
        elif self.number in (2, 3, 6, 7) and self.semitone_adjustment == 1:
            return True
        else:
            return False
    is_augmented = property(get_is_augmented)
    
    def get_is_diminished(self):
        # If you flat a perfect interval it becomes diminished
        if self.number in (1, 4, 5, 8) and self.semitone_adjustment == -1:
            return True
        else:
            return False
    is_diminished = property(get_is_diminished)

    def __unicode__(self):
        if self.num_semitones == 6:
            return u'Tritone'
        return u'%s %s' % (self.quality_name, self.size_name)

PERFECT_UNISON      = Interval(1)
DIMINISHED_2ND      = PERFECT_UNISON
MINOR_2ND           = Interval(2, -1)
AUGMENTED_UNISON    = MINOR_2ND
MAJOR_2ND           = Interval(2)
DIMINISHED_3RD      = MAJOR_2ND
MINOR_3RD           = Interval(3, -1)
AUGMENTED_2ND       = MINOR_3RD
MAJOR_3RD           = Interval(3)
DIMINISHED_4TH      = MAJOR_3RD
PERFECT_4TH         = Interval(4)
AUGMENTED_3RD       = PERFECT_4TH
TRITONE             = Interval(4, 1)
AUGMENTED_4TH       = TRITONE
DIMINISHED_5TH      = TRITONE
FLAT_5TH            = Interval(5, -1)
PERFECT_5TH         = Interval(5)
SHARP_5TH           = Interval(5, 1)
DIMINISHED_6TH      = PERFECT_5TH
MINOR_6TH           = Interval(6, -1)
AUGMENTED_5TH       = MINOR_6TH
MAJOR_6TH           = Interval(6)
DIMINISHED_7TH      = MAJOR_6TH
MINOR_7TH           = Interval(7, -1)
AUGMENTED_6TH       = MINOR_7TH
MAJOR_7TH           = Interval(7)
DIMINISHED_OCTAVE   = MAJOR_7TH
PERFECT_OCTAVE      = Interval(8)
AUGMENTED_7TH       = PERFECT_OCTAVE
MINOR_9TH           = Interval(9, -1)
MAJOR_9TH           = Interval(9)
SHARP_9TH           = Interval(9, 1)
SHARP_11TH          = Interval(11, -1)
MAJOR_11TH          = Interval(11)
MAJOR_13TH          = Interval(13)

INTERVALS = [
    PERFECT_UNISON,
    MINOR_2ND,
    MAJOR_2ND,
    MINOR_3RD,
    MAJOR_3RD,
    PERFECT_4TH,
    TRITONE,
    PERFECT_5TH,
    MINOR_6TH,
    MAJOR_6TH,
    MINOR_7TH,
    MAJOR_7TH,
    PERFECT_OCTAVE,
    AUGMENTED_7TH,
    MAJOR_9TH,
    SHARP_11TH,
    MAJOR_11TH,
    MAJOR_13TH
]

if __name__ == '__main__':
    # Test code
    for interval_num in range(8):
        interval = Interval(number=interval_num+1)
        print u'%s: %i semitones' % (interval, interval.num_semitones)
