import logging
import interval, note

class IntervalListNote(note.Note):
    def __init__(self, interval_list, note, interval):
        self.interval_list = interval_list
        self.interval = interval
        super(IntervalListNote, self).__init__(note.semitone_distance)

    def get_description(self):
        return u'%s: %s' % (
                super(IntervalListNote, self).__unicode__(), self.interval)
    description = property(get_description)


class InterValListFormula(object):
    def __init__(self, name='Unknown', interval_list=[]):
        self.name = name
        self.interval_list = []
        for interval in interval_list:
            self.add_interval(interval)

    def add_interval(self, interval):
        self.interval_list.append(interval)
        return interval

    def __unicode__(self):
        return unicode(self.name)

    def __hash__(self):
        return hash(self.name)

    def __cmp__(self, right):
        if hasattr(right, 'interval_list'):
            return cmp(self.interval_list, right.interval_list)
        return cmp(id(self), id(right))


class IntervalList(object):
    """
    An interval list formula applied to a root note

    @ivar type: the L{InterValListFormula} used for this list
    @ivar root: the root L{Note} for the list
    """
    NOTE_CLASS = IntervalListNote

    def __init__(self, type, root):
        self.type = type
        self.root = root

    def __unicode__(self):
        return u'%s %s %s' % (self.root, self.type, self.__class__.__name__)

    def __cmp__(self, right):
        return cmp(unicode(self), unicode(right))

    def __hash__(self):
        return hash(unicode(self))

    def get_note_list(self):
        if not self.root:
            return []
        note_list = []
        for interval in self.type.interval_list:
            note_list.append(self.__class__.NOTE_CLASS(
                        self, self.root + interval, interval))
        return note_list
    note_list = property(get_note_list)
    
    def get_description(self):
        result = u'=== %s ===\n' % self
        for note in self.note_list:
            result += u'%s\n' % note.description
        return result
    description = property(get_description)

    def similarity(self, target):
        return self.match_ratio(target) - self.miss_ratio(target)

    def match_ratio(self, target):
        match_list = []
        this_note_list = self.note_list
        target_note_list = target.note_list

        for note in this_note_list:
            if note in target_note_list:
                match_list.append(note)    

        return float(len(match_list)) / float(len(this_note_list))

    def miss_ratio(self, target):
        miss_list = []
        this_note_list = self.note_list
        target_note_list = target.note_list

        for note in this_note_list:
            if not note in target_note_list:
                miss_list.append(note)

        return float(len(miss_list)) / float(len(this_note_list))

