import logging

NOTE_NAMES = [
    u'C',
    u'C#/Db',
    u'D',
    u'D#/Eb',
    u'E',
    u'F',
    u'F#/Gb',
    u'G',
    u'G#/Ab',
    u'A',
    u'A#/Bb',
    u'B'
]

class Note(object):
    def __init__(self, semitone_distance=0):
        # semitone_distance is defined as number of semitones from middle C
        self.semitone_distance = semitone_distance

    def get_pitch_class(self):
        if self.semitone_distance >= 0:
            return self.semitone_distance % 12
        else:
            semitone_distance = self.semitone_distance
            while semitone_distance <= 0:
                semitone_distance += 12
            return semitone_distance
    pitch_class = property(get_pitch_class)

    def __unicode__(self):
        return NOTE_NAMES[self.pitch_class]    

    def __hash__(self):
        return self.pitch_class

    def __cmp__(self, right):
        if hasattr(right, 'semitone_distance'):
            return cmp(self.semitone_distance, right.semitone_distance)
        return cmp(id(self), id(right))

    def __add__(self, other):
        from interval import Interval
        if isinstance(other, Interval):
            semitone_distance = self.semitone_distance + other.num_semitones
        else:
            try:
                semitone_distance = self.semitone_distance + int(other)
            except:
                raise TypeError('Unable to add Note + %s' % 
                            other.__class__.__name__)
        new_note = Note(semitone_distance)
        logging.debug(u'%s of %s = %s' % (other, self, new_note))
        return new_note

C = Note(0)
C_SHARP = Note(1)
D_FLAT = C_SHARP
D = Note(2)
D_SHARP = Note(3)
E_FLAT = D_SHARP
E = Note(4)
F = Note(5)
F_SHARP = Note(6)
G_FLAT = F_SHARP
G = Note(7)
G_SHARP = Note(8)
A_FLAT = G_SHARP
A = Note(9)
A_SHARP = Note(10)
B_FLAT = A_SHARP
B = Note(11)

NOTES = [
    C, C_SHARP, D, D_SHARP, E, F, F_SHARP, G, G_SHARP, A, A_SHARP, B
]

NOTES_BY_NAME = {
    'c':    C,
    'c#':   C_SHARP,
    'db':   D_FLAT,
    'd':    D,
    'd#':   D_SHARP,
    'eb':   E_FLAT,
    'e':    E,
    'f':    F,
    'f#':   F_SHARP,
    'gb':   G_FLAT,
    'g':    G,
    'g#':   G_SHARP,
    'ab':   A_FLAT,
    'a':    A,
    'a#':   A_SHARP,
    'bb':   B_FLAT,
    'b':    B
}
