#!/usr/bin/env python
import logging
import interval, interval_list, note

class ScaleNote(interval_list.IntervalListNote):
    pass


class ScaleType(interval_list.InterValListFormula):
    def __init__(self, name='Unknown Scale', interval_list=[]):
        super(ScaleType, self).__init__(name=name, interval_list=interval_list)


class Scale(interval_list.IntervalList):
    pass


CHROMATIC = ScaleType('Chromatic', [
    interval.PERFECT_UNISON,
    interval.MINOR_2ND,
    interval.MAJOR_2ND,
    interval.MINOR_3RD,
    interval.MAJOR_3RD,
    interval.PERFECT_4TH,
    interval.TRITONE,
    interval.PERFECT_5TH,
    interval.MINOR_6TH,
    interval.MAJOR_6TH,
    interval.MINOR_7TH,
    interval.MAJOR_7TH,
    interval.PERFECT_OCTAVE
])

MAJOR = ScaleType('Major', [
    interval.PERFECT_UNISON,
    interval.MAJOR_2ND,
    interval.MAJOR_3RD,
    interval.PERFECT_4TH,
    interval.PERFECT_5TH,
    interval.MAJOR_6TH,
    interval.MAJOR_7TH,
    interval.PERFECT_OCTAVE
])

MINOR = ScaleType('Minor', [
    interval.PERFECT_UNISON,
    interval.MAJOR_2ND,
    interval.MINOR_3RD,
    interval.PERFECT_4TH,
    interval.PERFECT_5TH,
    interval.MINOR_6TH,
    interval.MINOR_7TH,
    interval.PERFECT_OCTAVE
])

PENTATONIC_MAJOR = ScaleType('Pentatonic Major', [
    interval.PERFECT_UNISON,
    interval.MAJOR_2ND,
    interval.MAJOR_3RD,
    interval.PERFECT_5TH,
    interval.MAJOR_6TH,
    interval.PERFECT_OCTAVE
])

PENTATONIC_MINOR = ScaleType('Pentatonic Minor', [
    interval.PERFECT_UNISON,
    interval.MINOR_3RD,
    interval.PERFECT_4TH,
    interval.PERFECT_5TH,
    interval.MINOR_7TH,
    interval.PERFECT_OCTAVE
])

PENTATONIC_MINOR = ScaleType('Pentatonic Minor', [
    interval.PERFECT_UNISON,
    interval.MINOR_3RD,
    interval.PERFECT_4TH,
    interval.PERFECT_5TH,
    interval.MINOR_7TH,
    interval.PERFECT_OCTAVE
])

BLUES = ScaleType('Blues', [
    interval.PERFECT_UNISON,
    interval.MINOR_3RD,
    interval.PERFECT_4TH,
    interval.DIMINISHED_5TH,
    interval.PERFECT_5TH,
    interval.MINOR_7TH,
    interval.PERFECT_OCTAVE
])

WHOLE_TONE = ScaleType('Whole-Tone', [
    interval.PERFECT_UNISON,
    interval.MAJOR_2ND,
    interval.MAJOR_3RD,
    interval.TRITONE,
    interval.MINOR_6TH,
    interval.MINOR_7TH,
    interval.PERFECT_OCTAVE
])

OCTATONIC = ScaleType('Octatonic', [
    interval.PERFECT_UNISON,
    interval.MINOR_2ND,
    interval.MINOR_3RD,
    interval.MAJOR_3RD,
    interval.TRITONE,
    interval.PERFECT_5TH,
    interval.MAJOR_6TH,
    interval.MINOR_7TH,
    interval.PERFECT_OCTAVE
])

DIMINISHED = ScaleType('Diminished', [
    interval.PERFECT_UNISON,
    interval.MAJOR_2ND,
    interval.MINOR_3RD,
    interval.PERFECT_4TH,
    interval.TRITONE,
    interval.MINOR_6TH,
    interval.MAJOR_6TH,
    interval.MAJOR_7TH,
    interval.PERFECT_OCTAVE
])

HARMONIC_MINOR = ScaleType('Harmonic Minor', [
    interval.PERFECT_UNISON,
    interval.MAJOR_2ND,
    interval.MINOR_3RD,
    interval.PERFECT_4TH,
    interval.PERFECT_5TH,
    interval.MINOR_6TH,
    interval.MAJOR_7TH,
    interval.PERFECT_OCTAVE
])

MELODIC_MINOR = ScaleType('Harmonic Minor', [
    interval.PERFECT_UNISON,
    interval.MAJOR_2ND,
    interval.MINOR_3RD,
    interval.PERFECT_4TH,
    interval.PERFECT_5TH,
    interval.MAJOR_6TH,
    interval.MAJOR_7TH,
    interval.PERFECT_OCTAVE
])

TYPES = [
    CHROMATIC,
    MAJOR,
    MINOR,
    PENTATONIC_MAJOR,
    PENTATONIC_MINOR,
    BLUES,
    WHOLE_TONE,
    OCTATONIC,
    DIMINISHED,
    HARMONIC_MINOR,
]

TYPES_BY_NAME = {
}

def get_all_scales():
    scale_list = []
    for root in note.NOTES:
        for type in TYPES:
            scale_list.append(Scale(type, root))
    return scale_list

def normalize_scale_name(name):
    name = name.strip()
    name = name.lower()
    name = name.replace(' ', '_')
    name = name.replace('/', '_')
    while '__' in name:
        name = name.replace('__', '_')
    return name

for type in TYPES:
    TYPES_BY_NAME[normalize_scale_name(type.name)] = type


if __name__ == '__main__':
    import sys

    if len(sys.argv) >= 3: 
        scale_type = TYPES_BY_NAME.get(sys.argv[2].lower(), CHROMATIC)
        root_note = note.NOTES_BY_NAME.get(sys.argv[1].lower(), note.C)
        scale = Scale(scale_type, root_note)
        print scale.description
    else:
        for scale_type in TYPES:
            for root in note.NOTES:
                scale = Scale(scale_type, root)
                print scale.description
