from PySide.QtGui import QApplication
from ui.qt4.pyfret import PyFret

import logging

class PyFretMaemo(PyFret):
    def __init__(self, *args, **kwargs):
        super(PyFretMaemo, self).__init__(*args, **kwargs)
        self.scale_selection_dialog = None
        self.chord_selection_dialog = None
        self.fretboard_selection_dialog = None

    def show_about_dialog(self):    
        from ui.maemoqt4.controllers.about_dialog import MaemoAboutDialog
        if hasattr(self.about_dialog, 'show'):
            self.about_dialog.show()
        else:
            self.about_dialog = MaemoAboutDialog(self)

    def select_scale_prompt(self):
        from ui.maemoqt4.controllers.scale_selection_dialog import ScaleSelectionDialog
        if hasattr(self.scale_selection_dialog, 'show'):
            self.scale_selection_dialog.show()
        else:
            self.scale_selection_dialog = ScaleSelectionDialog(self)
            self.scale_selection_dialog.show()
        if self.scale_selection_dialog.exec_() and self.fretboard_window:
            self.fretboard_window.current_scale = self.scale_selection_dialog.view.scale_chooser.scale
            self.fretboard_window.draw_selected_scale()

    def select_chord_prompt(self):
        from ui.maemoqt4.controllers.chord_selection_dialog import ChordSelectionDialog
        if hasattr(self.chord_selection_dialog, 'show'):
            self.chord_selection_dialog.show()
        else:
            self.chord_selection_dialog = ChordSelectionDialog(self)
            self.chord_selection_dialog.show()
        if self.chord_selection_dialog.exec_() and self.fretboard_window:
            self.fretboard_window.current_chord = self.chord_selection_dialog.view.chord_chooser.chord
            self.fretboard_window.draw_selected_chord()

    def select_fretboard_prompt(self):
        from ui.maemoqt4.controllers.fretboard_selection_dialog import FretboardSelectionDialog
        if hasattr(self.fretboard_selection_dialog, 'show'):
            self.fretboard_selection_dialog.show()
        else:
            self.fretboard_selection_dialog = FretboardSelectionDialog(self)
            self.fretboard_selection_dialog.show()
        if self.fretboard_selection_dialog.exec_() and self.fretboard_window:
            self.fretboard_window.view.fretboard_display.fretboard = self.fretboard_selection_dialog.view.fretboard_chooser.fretboard
            self.fretboard_window.display_interval_list(self.fretboard_window.cur_interval_list, force=True)
                        
    def init_ui_actions(self):
        from PySide.QtGui import QAction
        from PySide.QtCore import QObject, SIGNAL
        super(PyFretMaemo, self).init_ui_actions()
        self.ui_actions['select_scale'] = QAction(u'Select Scale', self)
        self.ui_actions['select_chord'] = QAction(u'Select Chord', self)
        QObject.connect(self.ui_actions['select_scale'], 
                SIGNAL('triggered()'),
                self.select_scale_prompt)
        QObject.connect(self.ui_actions['select_chord'], 
                SIGNAL('triggered()'),
                self.select_chord_prompt)

    def show_fretboard_window(self):
        from ui.maemoqt4.controllers.fretboard_window import MaemoFretboardWindow
        if hasattr(self.fretboard_window, 'show'):
            self.fretboard_window.show()
        else:
            self.fretboard_window = MaemoFretboardWindow(self)
            self.fretboard_window.show()


def main():
    import sys
    try:
        import music_theory
    except ImportError:
        sys.path.append('../')
    logging.basicConfig(level=logging.WARNING)
    app = PyFretMaemo(sys.argv)
    app.launch()
