import logging

from ui.qt4.controllers import Dialog
class ChordTypeEditDialog(Dialog):
    def __init__(self, app, *args, **kwargs):
        from PySide.QtCore import QSize
        from ui.qt4.views.chord_type_edit import ChordTypeEditView
        from music_theory.chord import MAJOR
        super(ChordTypeEditDialog, self).__init__(app, *args)
        self.setWindowTitle(u'Chord Formulae Editor')
        view = ChordTypeEditView(self)
        self.resize(QSize(400, 500))
        self.render(view)
        self.connect_signals()
        self.on_chord_type_change()

    def connect_signals(self):
        from PySide.QtCore import QObject, SIGNAL
        QObject.connect(self.view.chord_type_chooser, 
                SIGNAL('currentIndexChanged(int)'),
                self.on_chord_type_change)
        QObject.connect(self.view.save_btn, 
                SIGNAL('clicked()'), self.do_save)

    def init_widget_choices(self):
        for widget in ( [self.view.chord_type_chooser] + 
                        self.view.chord_type_editor.interval_list_editor.interval_editor_list ):
            widget.init_choices()

    def on_chord_type_change(self, **kwargs):
        chord_type = self.view.chord_type_chooser.chord_type
        self.view.chord_type_editor.chord_type = chord_type

    def do_save(self):
        from music_theory.chord import TYPES
        from PySide.QtGui import QMessageBox
        new_chord_type = self.view.chord_type_editor.chord_type
        for chord_type in TYPES:
            if chord_type.name == new_chord_type.name:
                QMessageBox.critical(None,
                        u'Unable to save chord formula',
                        u'You must specify a unique name for this chord formula')
                return
        TYPES.append(new_chord_type)
        self.app.init_widget_choices()
        self.view.chord_type_chooser.chord_type = new_chord_type

