import logging

from ui.qt4.controllers import Dialog
class FretboardEditDialog(Dialog):
    def __init__(self, app, *args, **kwargs):
        from PySide.QtCore import QSize
        from ui.qt4.views.fretboard_edit import FretboardEditView
        from music_theory.fretboard import STANDARD_GUITAR
        super(FretboardEditDialog, self).__init__(app, *args)
        self.setWindowTitle(u'Instrument/Tuning Editor')
        view = FretboardEditView(self)
        self.resize(QSize(400, 500))
        self.render(view)
        self.connect_signals()
        self.view.fretboard_chooser.fretboard = STANDARD_GUITAR

    def connect_signals(self):
        from PySide.QtCore import QObject, SIGNAL
        QObject.connect(self.view.fretboard_chooser,
                SIGNAL('currentIndexChanged(int)'),
                self.on_fretboard_change)
        QObject.connect(self.view.save_btn,
                SIGNAL('clicked()'), self.do_save)

    def init_widget_choices(self):
        for widget in ( [self.view.fretboard_chooser] + 
                        self.view.fretboard_editor.string_list_editor.string_editor_list ):
            widget.init_choices()

    def on_fretboard_change(self, **kwargs):
        fretboard = self.view.fretboard_chooser.fretboard
        self.view.fretboard_editor.fretboard = fretboard

    def do_save(self):
        from music_theory.fretboard import FRETBOARDS
        from PySide.QtGui import QMessageBox
        new_fretboard = self.view.fretboard_editor.fretboard
        for fretboard in FRETBOARDS:
            if fretboard.name == new_fretboard.name:
                QMessageBox.critical(None,
                        u'Unable to save fretboard',
                        u'You must specify a unique name for this fretboard')
                return
        FRETBOARDS.append(new_fretboard)
        self.app.init_widget_choices()
        self.view.fretboard_chooser.fretboard = new_fretboard

