import logging

from ui.qt4.controllers import Dialog
class ScaleTypeEditDialog(Dialog):
    def __init__(self, app, *args, **kwargs):
        from PySide.QtCore import QSize
        from ui.qt4.views.scale_type_edit import ScaleTypeEditView
        from music_theory.scale import MAJOR
        super(ScaleTypeEditDialog, self).__init__(app, *args)
        self.setWindowTitle(u'Scale Formulae Editor')
        view = ScaleTypeEditView(self)
        self.resize(QSize(400, 500))
        self.render(view)
        self.connect_signals()
        self.view.scale_type_chooser.scale_type = MAJOR

    def connect_signals(self):
        from PySide.QtCore import QObject, SIGNAL
        QObject.connect(self.view.scale_type_chooser, 
                SIGNAL('currentIndexChanged(int)'),
                self.on_scale_type_change)
        QObject.connect(self.view.save_btn, 
                SIGNAL('clicked()'), self.do_save)

    def init_widget_choices(self):
        for widget in ( [self.view.scale_type_chooser] + 
                        self.view.scale_type_editor.interval_list_editor.interval_editor_list ):
            widget.init_choices()

    def on_scale_type_change(self, **kwargs):
        scale_type = self.view.scale_type_chooser.scale_type
        self.view.scale_type_editor.scale_type = scale_type

    def do_save(self):
        from music_theory.scale import TYPES
        from PySide.QtGui import QMessageBox
        new_scale_type = self.view.scale_type_editor.scale_type
        for scale_type in TYPES:
            if scale_type.name == new_scale_type.name:
                QMessageBox.critical(None,
                        u'Unable to save scale type',
                        u'You must specify a unique name for this scale type')
                return
        TYPES.append(new_scale_type)
        self.app.init_widget_choices()
        self.view.scale_type_chooser.scale_type = new_scale_type

