from PySide.QtGui import QApplication
import logging

class PyFret(QApplication):
    def launch(self):  
        self.fretboard_window = None
        self.about_dialog = None
        self.fretboard_edit_dialog = None
        self.scale_type_edit_dialog = None
        self.chord_type_edit_dialog = None
        self.init_ui_actions()
        self.show_fretboard_window()
        self.exec_()

    def show_about_dialog(self):    
        from ui.qt4.controllers.about_dialog import AboutDialog
        if hasattr(self.about_dialog, 'show'):
            self.about_dialog.show()
        else:
            self.about_dialog = AboutDialog(self)

    def select_fretboard_prompt(self):
        pass

    def show_fretboard_window(self):
        from ui.qt4.controllers.fretboard_window import FretboardWindow
        if hasattr(self.fretboard_window, 'show'):
            self.fretboard_window.show()
        else:
            self.fretboard_window = FretboardWindow(self)
            self.fretboard_window.show()

    def show_scale_type_edit_dialog(self):
        from ui.qt4.controllers.scale_type_edit_dialog import ScaleTypeEditDialog
        if hasattr(self.scale_type_edit_dialog, 'show'):
            self.scale_type_edit_dialog.show()
        else:
            self.scale_type_edit_dialog = ScaleTypeEditDialog(self, self.fretboard_window)
            self.scale_type_edit_dialog.show()

    def show_chord_type_edit_dialog(self):
        from ui.qt4.controllers.chord_type_edit_dialog import ChordTypeEditDialog
        if hasattr(self.chord_type_edit_dialog, 'show'):
            self.chord_type_edit_dialog.show()
        else:
            self.chord_type_edit_dialog = ChordTypeEditDialog(self, self.fretboard_window)
            self.chord_type_edit_dialog.show()

    def show_fretboard_edit_dialog(self):
        from ui.qt4.controllers.fretboard_edit_dialog import FretboardEditDialog
        if hasattr(self.fretboard_edit_dialog, 'show'):
            self.fretboard_edit_dialog.show()
        else:
            self.fretboard_edit_dialog = FretboardEditDialog(self, self.fretboard_window)
            self.fretboard_edit_dialog.show()

    def init_ui_actions(self):
        from PySide.QtGui import QAction
        from PySide.QtCore import QObject, SIGNAL
        self.ui_actions = {
            'quit':                         QAction(u'Quit', self),
            'select_fretboard':             QAction(u'Instrument/Tuning', self),
            'show_about_dialog':            QAction(u'About PyFret', self),
            'show_scale_type_edit_dialog':  QAction(u'Scale Formulae...', self),
            'show_chord_type_edit_dialog':  QAction(u'Chord Formulae...', self),
            'show_fretboard_edit_dialog':   QAction(u'Instruments/Tunings...', self)
        }

        QObject.connect(self.ui_actions['show_scale_type_edit_dialog'],
                SIGNAL('triggered()'),
                self.show_scale_type_edit_dialog)
        QObject.connect(self.ui_actions['show_chord_type_edit_dialog'],
                SIGNAL('triggered()'),
                self.show_chord_type_edit_dialog)
        QObject.connect(self.ui_actions['show_fretboard_edit_dialog'],
                SIGNAL('triggered()'),
                self.show_fretboard_edit_dialog)
        QObject.connect(self.ui_actions['quit'], 
                SIGNAL('triggered()'), self.quit)
        QObject.connect(self.ui_actions['show_about_dialog'], 
                SIGNAL('triggered()'), self.show_about_dialog)
        QObject.connect(self.ui_actions['select_fretboard'], 
                SIGNAL('triggered()'), self.select_fretboard_prompt)

    def init_widget_choices(self):
        for controller in ( self.fretboard_window, 
                            self.scale_type_edit_dialog,
                            self.chord_type_edit_dialog,
                            self.fretboard_edit_dialog):
            if hasattr(controller, 'init_widget_choices'):
                controller.init_widget_choices()

    def quit(self):
        for window in (self.fretboard_window, self.scale_type_edit_dialog):
            if hasattr(window, 'close'):
                window.close()
            elif hasattr(window, 'reject'):
                window.reject()
        

def main():
    import sys
    try:
        import music_theory
    except ImportError:
        sys.path.append('../')
    logging.basicConfig(level=logging.WARNING)
    app = PyFret(sys.argv)
    app.launch()
