import logging

from ui.qt4.views import View

SCALE_DRAW_TAB_IDX = 0
CHORD_DRAW_TAB_IDX = 1
DISPLAY_HISTORY_TAB_IDX = 2

DISPLAY_INFORMATION_TAB_IDX = 0
DISPLAY_SCALE_SIMILARITY_TAB_IDX = 1
DISPLAY_CHORD_SIMILARITY_TAB_IDX = 2


class FretboardView(View):
    def __init__(self, controller):
        super(FretboardView, self).__init__(controller)
        self.init_widgets()
        self.init_layout()

    def init_widgets(self):    
        from PySide.QtGui import QLabel, QFrame
        from PySide.QtGui import QTabWidget, QPushButton, QListWidget, QTextEdit
        from ui.qt4.widgets.scale_chooser import ScaleChooser
        from ui.qt4.widgets.chord_chooser import ChordChooser
        from ui.qt4.widgets.fretboard_display import FretboardDisplay
        from ui.qt4.widgets.fretboard_chooser import FretboardChooser
        from ui.qt4.widgets.scale_similarity_table import ScaleSimilarityTable
        from ui.qt4.widgets.chord_similarity_table import ChordSimilarityTable
        from ui.qt4.widgets.interval_list_info import IntervalListInfo
        from ui.qt4.widgets.chord_type_editor import ChordTypeEditor
        from ui.qt4.widgets.scale_type_editor import ScaleTypeEditor
        #self.fretboard_chooser_lbl = QLabel()
        #self.fretboard_chooser_lbl.setText(u'Instrument/Tuning: ')

        self.scale_draw_tab = QFrame()
        self.scale_chooser = ScaleChooser()
        self.scale_draw_btn = QPushButton()
        self.scale_draw_btn.setText(u'Draw Scale')

        self.chord_draw_tab = QFrame()
        self.chord_chooser = ChordChooser()
        self.chord_draw_btn = QPushButton()
        self.chord_draw_btn.setText(u'Draw Chord')

        self.fretboard_draw_tab = QFrame()
        self.fretboard_chooser = FretboardChooser()
        self.fretboard_display = FretboardDisplay()

        self.clear_fretboard_btn = QPushButton()
        self.clear_fretboard_btn.setText(u'Clear')

        self.zoom_out_btn = QPushButton()
        self.zoom_out_btn.setText(u'Zoom Out')

        self.zoom_in_btn = QPushButton()
        self.zoom_in_btn.setText(u'Zoom In')

        self.display_information_interval_list = IntervalListInfo()
        #self.display_information_txt = QTextEdit()
        #self.display_information_txt.setReadOnly(True)
        self.display_scale_similarity_tab = QFrame()
        self.scale_similarity_table = ScaleSimilarityTable()
        self.display_chord_similarity_tab = QFrame()
        self.chord_similarity_table = ChordSimilarityTable()

        self.display_history_tab = QFrame()
        self.display_history_lst = QListWidget()

        self.display_history_remove_btn = QPushButton()
        self.display_history_remove_btn.setText(u'Remove')

        self.display_history_clear_btn = QPushButton()
        self.display_history_clear_btn.setText(u'Clear')
        self.selector_pane = QFrame()


    def init_layout(self):  
        from PySide.QtGui import QVBoxLayout, QHBoxLayout, QTabWidget
        from PySide.QtGui import QSizePolicy, QSpacerItem
        self.main_layout = QVBoxLayout(self)
        self.draw_tab_widget = QTabWidget()
        self.display_tab_widget = QTabWidget()

        self.selector_pane.setSizePolicy(QSizePolicy(
                    QSizePolicy.Expanding, QSizePolicy.Expanding))
        self.fretboard_chooser.setSizePolicy(QSizePolicy(
                    QSizePolicy.Expanding, QSizePolicy.Minimum))
        self.zoom_out_btn.setSizePolicy(QSizePolicy(
                    QSizePolicy.Minimum, QSizePolicy.Minimum))
        self.zoom_in_btn.setSizePolicy(QSizePolicy(
                    QSizePolicy.Minimum, QSizePolicy.Minimum))

        self.selector_layout = QHBoxLayout(self.selector_pane)
        self.scale_draw_layout = QVBoxLayout(self.scale_draw_tab)
        self.scale_draw_btn_layout = QHBoxLayout()
        self.chord_draw_layout = QVBoxLayout(self.chord_draw_tab)
        self.chord_draw_btn_layout = QHBoxLayout()
        self.fretboard_draw_layout = QVBoxLayout(self.fretboard_draw_tab)
        self.fretboard_draw_btn_layout = QHBoxLayout()
        self.display_history_layout = QVBoxLayout(self.display_history_tab)
        self.display_history_btn_layout = QHBoxLayout()
        self.scale_similarity_layout = QVBoxLayout(self.display_scale_similarity_tab)
        self.chord_similarity_layout = QVBoxLayout(self.display_chord_similarity_tab)
        self.btn_layout = QHBoxLayout()

        self.scale_draw_layout.addWidget(self.scale_chooser)
        self.scale_draw_layout.addLayout(self.scale_draw_btn_layout)
        scale_btn_spacer = QSpacerItem(1, 1, QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.scale_draw_btn_layout.addItem(scale_btn_spacer)
        self.scale_draw_btn_layout.addWidget(self.scale_draw_btn)
        self.chord_draw_layout.addWidget(self.chord_chooser)
        self.chord_draw_layout.addLayout(self.chord_draw_btn_layout)
        chord_btn_spacer = QSpacerItem(1, 1, QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.chord_draw_btn_layout.addItem(chord_btn_spacer)
        self.chord_draw_btn_layout.addWidget(self.chord_draw_btn)
        self.draw_tab_widget.addTab(self.scale_draw_tab, u'Scales')
        self.draw_tab_widget.addTab(self.chord_draw_tab, u'Chords')

        self.fretboard_draw_layout.addWidget(self.fretboard_chooser)
        fretboard_chooser_spacer = QSpacerItem(1, 1, QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.fretboard_draw_layout.addItem(fretboard_chooser_spacer)
        self.fretboard_draw_layout.addLayout(self.fretboard_draw_btn_layout)
        self.fretboard_draw_btn_layout.addWidget(self.zoom_out_btn)
        self.fretboard_draw_btn_layout.addWidget(self.clear_fretboard_btn)
        self.fretboard_draw_btn_layout.addWidget(self.zoom_in_btn)
        self.draw_tab_widget.addTab(self.fretboard_draw_tab, u'Fretboard')

        #self.information_layout.addWidget(self.display_information_txt)
        self.display_tab_widget.addTab(self.display_information_interval_list, u'Intervals')

        self.display_tab_widget.setSizePolicy(QSizePolicy(
                    QSizePolicy.Expanding, QSizePolicy.Expanding))

        self.scale_similarity_layout.addWidget(self.scale_similarity_table)
        self.display_tab_widget.addTab(self.display_scale_similarity_tab, u'Similar Scales')

        self.chord_similarity_layout.addWidget(self.chord_similarity_table)
        self.display_tab_widget.addTab(self.display_chord_similarity_tab, u'Similar Chords')


        self.display_history_layout.addWidget(self.display_history_lst)
        self.display_history_layout.addLayout(self.display_history_btn_layout)
        history_btn_spacer = QSpacerItem(1, 1, QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.display_history_btn_layout.addItem(history_btn_spacer)
        self.display_history_btn_layout.addWidget(self.display_history_remove_btn)
        self.display_history_btn_layout.addWidget(self.display_history_clear_btn)
        self.draw_tab_widget.addTab(self.display_history_tab, u'History')


        self.main_layout.addWidget(self.fretboard_display)

        self.main_layout.addWidget(self.selector_pane)
        #selector_spacer = QSpacerItem(1, 1, QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.selector_layout.addWidget(self.draw_tab_widget)
        #self.selector_layout.addItem(selector_spacer)
        self.selector_layout.addWidget(self.display_tab_widget)


