import logging

from PySide.QtGui import QGroupBox
class ChordChooser(QGroupBox):
    def __init__(self, *args, **kwargs):
        from PySide.QtGui import QVBoxLayout, QHBoxLayout, QLabel
        from PySide.QtGui import QSpacerItem, QSizePolicy
        from ui.qt4.widgets.chord_type_chooser import ChordTypeChooser
        from ui.qt4.widgets.key_chooser import KeyChooser
        super(ChordChooser, self).__init__(*args)
        self.chord_type_lbl = QLabel()
        self.chord_type_lbl.setText(u'Type: ')
        self.chord_type_chooser = ChordTypeChooser()
        self.chord_type_chooser.setSizePolicy(QSizePolicy(
                    QSizePolicy.Expanding, QSizePolicy.Minimum))

        self.key_lbl = QLabel()
        self.key_lbl.setText(u'Key: ')
        self.key_chooser = KeyChooser()
        self.key_chooser.setSizePolicy(QSizePolicy(
                    QSizePolicy.Expanding, QSizePolicy.Minimum))

        self.main_layout = QVBoxLayout(self)
        self.chord_type_layout = QHBoxLayout()
        self.key_layout = QHBoxLayout()

        self.main_layout.addLayout(self.chord_type_layout)
        self.chord_type_layout.addWidget(self.chord_type_lbl)
        self.chord_type_layout.addWidget(self.chord_type_chooser)

        self.main_layout.addLayout(self.key_layout)
        self.key_layout.addWidget(self.key_lbl)
        self.key_layout.addWidget(self.key_chooser)

        spacer = QSpacerItem(1, 1, QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.main_layout.addItem(spacer)

        self.connect_signals()

    def init_choices(self):
        for widget in (self.key_chooser, self.chord_type_chooser):
            widget.init_choices()

    def connect_signals(self):
        from PySide.QtCore import QObject, SIGNAL
        QObject.connect(self.key_chooser, 
                SIGNAL('currentIndexChanged(int)'),
                self.on_change)
        QObject.connect(self.chord_type_chooser, 
                SIGNAL('currentIndexChanged(int)'),
                self.on_change)

    def on_change(self):
        from PySide.QtCore import SIGNAL
        self.emit(SIGNAL('selectedChordChanged'))

    def get_chord(self):
        from music_theory.chord import Chord
        key = self.key_chooser.key
        chord_type = self.chord_type_chooser.chord_type

        if key and chord_type:
            return Chord(chord_type, key)

    def set_chord(self, new_chord):
        if not new_chord:
            return
        self.chord_type_chooser.chord_type = new_chord.type
        self.key_chooser.key = new_chord.root
        return self.chord
    chord = property(get_chord, set_chord)

