import logging
from PySide.QtGui import QTableWidgetItem
class ChordSimilarityTableItem(QTableWidgetItem):
    def __init__(self, chord_item, text, parent):
        self.chord_item = chord_item
        self.text = text
        logging.info('%r' % self)
        super(ChordSimilarityTableItem, self).__init__(unicode(self.text))

    def __repr__(self):
        return "%s(%s, %r)" % (
                self.__class__.__name__, self.chord_item, self.text)

from PySide.QtGui import QTableWidget
class ChordSimilarityTable(QTableWidget):
    def __init__(self, *args, **kwargs):
        self._target = None
        super(ChordSimilarityTable, self).__init__(*args)
        #self.setColumnCount(4)
        self.setSortingEnabled(False)
        self.setColumnCount(4)
        self.set_headers()
        #self.setSelectionMode(QListWidget.NoSelection)

    def set_headers(self):
        from PySide.QtGui import QHeaderView
        self.setHorizontalHeaderLabels(['Chord Name', 'Total', 'Match', 'Miss'])
        self.verticalHeader().setVisible(False)
        self.horizontalHeader().resizeSections(QHeaderView.ResizeToContents)
        self.horizontalHeader().setResizeMode(QHeaderView.Interactive)

    def get_target(self):
        return self._target

    def set_target(self, new_target):
        if new_target != self.target:
            from music_theory.chord import get_all_chords
            from PySide.QtCore import Qt
            self.clear()
            self._target = new_target

            chord_list = []

            for chord in get_all_chords():
                match = chord.match_ratio(self.target)
                miss = chord.miss_ratio(self.target)
                similarity = match - miss
                chord_list.append((chord, similarity, match, miss))


            self.setRowCount(len(chord_list))

            row = 0
            for chord_tuple in chord_list:
                chord, similarity, match, miss = chord_tuple
                name = unicode(chord)

                name_item = ChordSimilarityTableItem(chord, name, self)
                self.setItem(row, 0, name_item)

                similarity_item = ChordSimilarityTableItem(chord, similarity, self)
                self.setItem(row, 1, similarity_item)

                match_item = ChordSimilarityTableItem(chord, match, self)
                self.setItem(row, 2, match_item)

                miss_item = ChordSimilarityTableItem(chord, miss, self)
                self.setItem(row, 3, miss_item)
                row += 1
            self.setSortingEnabled(True)
            self.sortByColumn(1, Qt.DescendingOrder)

        self.set_headers()
        return self.target
    target = property(get_target, set_target)        


