import logging

from PySide.QtGui import QGroupBox
class ChordTypeEditor(QGroupBox):
    def __init__(self, *args, **kwargs):
        from PySide.QtGui import QVBoxLayout, QHBoxLayout, QSpacerItem
        from PySide.QtGui import QLineEdit, QLabel, QSizePolicy, QPushButton
        from PySide.QtCore import Qt
        from ui.qt4.widgets.interval_list_editor import IntervalListEditor
        super(ChordTypeEditor, self).__init__(*args)
        self.name_label = QLabel()
        self.name_label.setText(u'Name: ')
        self.name_edit = QLineEdit()

        self.symbol_label = QLabel()
        self.symbol_label.setText(u'Symbol: ')
        self.symbol_edit = QLineEdit()

        #self.interval_list_lbl = QLabel()
        #self.interval_list_lbl.setText(u'Interval List')
        #self.interval_list_lbl.setAlignment(Qt.AlignCenter)
        self.interval_list_editor = IntervalListEditor()

        self.main_layout = QVBoxLayout(self)
        self.name_layout = QHBoxLayout()
        self.symbol_layout = QHBoxLayout()

        self.add_interval_btn = QPushButton()
        self.add_interval_btn.setText(u'Add Interval')

        self.main_layout.addLayout(self.name_layout)
        self.name_layout.addWidget(self.name_label)
        self.name_layout.addWidget(self.name_edit)
        self.main_layout.addLayout(self.symbol_layout)
        self.symbol_layout.addWidget(self.symbol_label)
        self.symbol_layout.addWidget(self.symbol_edit)
        #self.main_layout.addWidget(self.interval_list_lbl)
        self.main_layout.addWidget(self.interval_list_editor)
        interval_spacer = QSpacerItem(1, 1, QSizePolicy.Minimum, QSizePolicy.Minimum)
        self.main_layout.addItem(interval_spacer)
        self.main_layout.addWidget(self.add_interval_btn)

        self.connect_signals()

    def connect_signals(self):
        from PySide.QtCore import SIGNAL, QObject
        QObject.connect(self.add_interval_btn, SIGNAL('clicked()'),
                self.interval_list_editor.add_interval_editor)

    def get_chord_type(self):
        from music_theory.chord import ChordType
        interval_list = []
        for editor in self.interval_list_editor.interval_editor_list:
            interval_list.append(editor.interval_chooser.interval)
        name = unicode(self.name_edit.text())
        symbol = unicode(self.symbol_edit.text())
        return ChordType(name, symbol, interval_list)

    def set_chord_type(self, new_chord_type):
        if not new_chord_type:
            return
        self.name_edit.setText(new_chord_type.name)
        self.symbol_edit.setText(new_chord_type.symbol)
        interval_list = list(new_chord_type.interval_list)
        self.interval_list_editor.clear_interval_editor_list()
        for interval in interval_list:
            self.interval_list_editor.add_interval_editor(interval)
        return self.chord_type
    chord_type = property(get_chord_type, set_chord_type)


