import logging

from PySide.QtGui import QGroupBox
class FretboardEditor(QGroupBox):
    def __init__(self, *args, **kwargs):
        from PySide.QtGui import QVBoxLayout, QHBoxLayout, QSpacerItem, QSpinBox
        from PySide.QtGui import QLineEdit, QLabel, QSizePolicy, QPushButton
        from PySide.QtCore import Qt
        from ui.qt4.widgets.string_list_editor import StringListEditor
        from music_theory.fretboard import STANDARD_GUITAR
        super(FretboardEditor, self).__init__(*args)
        self.name_label = QLabel()
        self.name_label.setText(u'Name: ')
        self.name_edit = QLineEdit()

        self.num_frets_lbl = QLabel()
        self.num_frets_lbl.setText(u'Num Frets:')

        self.num_frets_spin = QSpinBox()

        self.string_list_lbl = QLabel()
        self.string_list_lbl.setText(u'String List')
        self.string_list_lbl.setAlignment(Qt.AlignCenter)
        self.string_list_editor = StringListEditor()

        self.main_layout = QVBoxLayout(self)
        self.name_layout = QHBoxLayout()
        self.num_frets_layout = QHBoxLayout()

        self.add_string_btn = QPushButton()
        self.add_string_btn.setText(u'Add String')

        self.main_layout.addLayout(self.name_layout)
        self.name_layout.addWidget(self.name_label)
        self.name_layout.addWidget(self.name_edit)
        self.main_layout.addLayout(self.num_frets_layout)
        self.num_frets_layout.addWidget(self.num_frets_lbl)
        self.num_frets_layout.addWidget(self.num_frets_spin)
        self.main_layout.addWidget(self.string_list_lbl)
        self.main_layout.addWidget(self.string_list_editor)
        string_spacer = QSpacerItem(1, 1, QSizePolicy.Minimum, QSizePolicy.Minimum)
        self.main_layout.addItem(string_spacer)
        self.main_layout.addWidget(self.add_string_btn)

        self.connect_signals()
        self.fretboard = STANDARD_GUITAR

    def connect_signals(self):
        from PySide.QtCore import SIGNAL, QObject
        QObject.connect(self.add_string_btn, SIGNAL('clicked()'),
                self.string_list_editor.add_string_editor)

    def get_fretboard(self):
        from music_theory.fretboard import Fretboard
        tuning = []
        for editor in self.string_list_editor.string_editor_list:
            tuning.append(editor.key_chooser.key)
        name = unicode(self.name_edit.text())
        num_frets = self.num_frets_spin.value()
        return Fretboard(tuning, name, num_frets=num_frets)

    def set_fretboard(self, new_fretboard):
        if not new_fretboard:
            return
        self.name_edit.setText(new_fretboard.name)
        tuning = list(new_fretboard.tuning)
        self.string_list_editor.clear_string_editor_list()
        for key in tuning:
            self.string_list_editor.add_string_editor(key)
        self.num_frets_spin.setValue(new_fretboard.num_frets)
        return self.fretboard
    fretboard = property(get_fretboard, set_fretboard)
        
