from PySide.QtGui import QFrame
class IntervalEditor(QFrame):
    def __init__(self, list_editor, *args, **kwargs):
        from ui.qt4.widgets.interval_chooser import IntervalChooser
        from PySide.QtGui import QHBoxLayout, QPushButton , QSizePolicy
        from PySide.QtCore import SIGNAL, QObject
        super(IntervalEditor, self).__init__(*args)
        self.list_editor = list_editor
        self.interval_chooser = IntervalChooser()
        self.interval_chooser.setSizePolicy(QSizePolicy(
                    QSizePolicy.Expanding, QSizePolicy.Minimum))
        self.remove_btn = QPushButton()
        self.remove_btn.setSizePolicy(QSizePolicy(
                    QSizePolicy.Minimum, QSizePolicy.Minimum))
        self.remove_btn.setText(u'Remove')
        self.main_layout = QHBoxLayout(self)

        self.main_layout.addWidget(self.interval_chooser)
        self.main_layout.addWidget(self.remove_btn)
        QObject.connect(self.remove_btn, SIGNAL('clicked()'),
                self.on_remove_click)

    def init_choices(self):
        self.interval_chooser.init_choices()

    def on_remove_click(self, **kwargs):
        self.list_editor.remove_interval_editor(self)


from music_theory.interval import PERFECT_UNISON
from PySide.QtGui import QScrollArea
class IntervalListEditor(QScrollArea):
    def __init__(self, *args):
        from PySide.QtGui import QFrame, QVBoxLayout, QSizePolicy, QSpacerItem
        super(IntervalListEditor, self).__init__(*args)
        self.setWidgetResizable(True)
        self.setSizePolicy(QSizePolicy(
                    QSizePolicy.Expanding, QSizePolicy.Expanding))
        self.interval_list_widget = QFrame()
        self.setWidget(self.interval_list_widget)

        self.interval_editor_list = []
        self.main_widget_layout = QVBoxLayout(self.interval_list_widget)
        self.interval_list_layout = QVBoxLayout()

        self.main_widget_layout.addLayout(self.interval_list_layout)
        interval_spacer = QSpacerItem(1, 1, QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.main_widget_layout.addItem(interval_spacer)

    def remove_interval_editor(self, interval_editor):
        self.interval_list_layout.removeWidget(interval_editor)
        self.interval_editor_list.remove(interval_editor)
        interval_editor.close()
        interval_editor.deleteLater()

    def add_interval_editor(self, interval=PERFECT_UNISON):
        editor = IntervalEditor(self)
        editor.interval_chooser.interval = interval
        self.interval_list_layout.addWidget(editor)
        self.interval_editor_list.append(editor)
        editor.show()

    def clear_interval_editor_list(self):
        for editor in list(self.interval_editor_list):
            self.remove_interval_editor(editor)

