import logging

from PySide.QtGui import QGroupBox
class ScaleChooser(QGroupBox):
    def __init__(self, *args, **kwargs):
        from PySide.QtGui import QVBoxLayout, QHBoxLayout, QLabel
        from PySide.QtGui import QSpacerItem, QSizePolicy
        from ui.qt4.widgets.scale_type_chooser import ScaleTypeChooser
        from ui.qt4.widgets.key_chooser import KeyChooser
        super(ScaleChooser, self).__init__(*args)
        self.scale_type_lbl = QLabel()
        self.scale_type_lbl.setText(u'Type: ')
        self.scale_type_chooser = ScaleTypeChooser()
        self.scale_type_chooser.setSizePolicy(QSizePolicy(
                    QSizePolicy.Expanding, QSizePolicy.Minimum))

        self.key_lbl = QLabel()
        self.key_lbl.setText(u'Key: ')
        self.key_chooser = KeyChooser()
        self.key_chooser.setSizePolicy(QSizePolicy(
                    QSizePolicy.Expanding, QSizePolicy.Minimum))

        self.main_layout = QVBoxLayout(self)
        self.scale_type_layout = QHBoxLayout()
        self.key_layout = QHBoxLayout()

        self.main_layout.addLayout(self.scale_type_layout)
        self.scale_type_layout.addWidget(self.scale_type_lbl)
        self.scale_type_layout.addWidget(self.scale_type_chooser)

        self.main_layout.addLayout(self.key_layout)
        self.key_layout.addWidget(self.key_lbl)
        self.key_layout.addWidget(self.key_chooser)

        spacer = QSpacerItem(1, 1, QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.main_layout.addItem(spacer)

        self.connect_signals()

    def init_choices(self):
        for widget in (self.key_chooser, self.scale_type_chooser):
            widget.init_choices()

    def connect_signals(self):
        from PySide.QtCore import QObject, SIGNAL
        QObject.connect(self.key_chooser, 
                SIGNAL('currentIndexChanged(int)'),
                self.on_change)
        QObject.connect(self.scale_type_chooser, 
                SIGNAL('currentIndexChanged(int)'),
                self.on_change)

    def on_change(self, *args):
        from PySide.QtCore import SIGNAL
        self.emit(SIGNAL('selectedScaleChanged'))

    def get_scale(self):
        from music_theory.scale import Scale
        key = self.key_chooser.key
        scale_type = self.scale_type_chooser.scale_type

        if key and scale_type:
            return Scale(scale_type, key)

    def set_scale(self, new_scale):
        if not new_scale:
            return
        self.scale_type_chooser.scale_type = new_scale.type
        self.key_chooser.key = new_scale.root
        return self.scale
    scale = property(get_scale, set_scale)

