import logging
from PySide.QtGui import QTableWidgetItem
class ScaleSimilarityTableItem(QTableWidgetItem):
    def __init__(self, scale_item, text, parent):
        self.scale_item = scale_item
        self.text = text
        logging.info('%r' % self)
        super(ScaleSimilarityTableItem, self).__init__(unicode(self.text))

    def __repr__(self):
        return "%s(%s, %r)" % (
                self.__class__.__name__, self.scale_item, self.text)

from PySide.QtGui import QTableWidget
class ScaleSimilarityTable(QTableWidget):
    def __init__(self, *args, **kwargs):
        self._target = None
        super(ScaleSimilarityTable, self).__init__(*args)
        self.setSortingEnabled(False)
        self.setColumnCount(4)
        self.set_headers()
        #self.setSelectionMode(QListWidget.NoSelection)

    def set_headers(self):
        from PySide.QtGui import QHeaderView
        self.setHorizontalHeaderLabels(['Scale Name', 'Total', 'Match', 'Miss'])
        self.verticalHeader().setVisible(False)
        self.horizontalHeader().resizeSections(QHeaderView.ResizeToContents)
        self.horizontalHeader().setResizeMode(QHeaderView.Interactive)

    def get_target(self):
        return self._target

    def set_target(self, new_target):
        if new_target != self.target:
            from music_theory.scale import get_all_scales
            from PySide.QtCore import Qt
            self.clear()
            self._target = new_target

            scale_list = []

            for scale in get_all_scales():
                match = scale.match_ratio(self.target)
                miss = scale.miss_ratio(self.target)
                similarity = match - miss
                scale_list.append((scale, similarity, match, miss))

            self.setRowCount(len(scale_list))

            row = 0
            for scale_tuple in scale_list:
                scale, similarity, match, miss = scale_tuple
                name = unicode(scale) 

                name_item = ScaleSimilarityTableItem(scale, name, self)
                self.setItem(row, 0, name_item)

                similarity_item = ScaleSimilarityTableItem(scale, similarity, self)
                self.setItem(row, 1, similarity_item)

                match_item = ScaleSimilarityTableItem(scale, match, self)
                self.setItem(row, 2, match_item)

                miss_item = ScaleSimilarityTableItem(scale, miss, self)
                self.setItem(row, 3, miss_item)
                row += 1
            self.setSortingEnabled(True)
            self.sortByColumn(1, Qt.DescendingOrder)
            #if item.scale_item == self.target:
            #    self.setCurrentItem(item)
        self.set_headers()
        return self.target     
    target = property(get_target, set_target)
