from PySide.QtGui import QFrame
class StringEditor(QFrame):
    def __init__(self, list_editor, *args, **kwargs):
        from ui.qt4.widgets.key_chooser import KeyChooser
        from PySide.QtGui import QHBoxLayout, QPushButton , QSizePolicy
        from PySide.QtCore import SIGNAL, QObject
        super(StringEditor, self).__init__(*args)
        self.list_editor = list_editor
        self.key_chooser = KeyChooser()
        self.key_chooser.setSizePolicy(QSizePolicy(
                    QSizePolicy.Expanding, QSizePolicy.Minimum))
        self.remove_btn = QPushButton()
        self.remove_btn.setSizePolicy(QSizePolicy(
                    QSizePolicy.Minimum, QSizePolicy.Minimum))
        self.remove_btn.setText(u'Remove')
        self.main_layout = QHBoxLayout(self)

        self.main_layout.addWidget(self.key_chooser)
        self.main_layout.addWidget(self.remove_btn)
        QObject.connect(self.remove_btn, SIGNAL('clicked()'),
                self.on_remove_click)

    def init_choices(self):
        self.key_chooser.init_choices()

    def on_remove_click(self, **kwargs):
        self.list_editor.remove_string_editor(self)


from music_theory.note import E
from PySide.QtGui import QScrollArea
class StringListEditor(QScrollArea):
    def __init__(self, *args):
        from PySide.QtGui import QFrame, QVBoxLayout, QSizePolicy, QSpacerItem
        super(StringListEditor, self).__init__(*args)
        self.setWidgetResizable(True)
        self.setSizePolicy(QSizePolicy(
                    QSizePolicy.Expanding, QSizePolicy.Expanding))
        self.string_list_widget = QFrame()
        self.setWidget(self.string_list_widget)

        self.string_editor_list = []
        self.main_widget_layout = QVBoxLayout(self.string_list_widget)
        self.string_list_layout = QVBoxLayout()

        self.main_widget_layout.addLayout(self.string_list_layout)
        string_spacer = QSpacerItem(1, 1, QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.main_widget_layout.addItem(string_spacer)

    def remove_string_editor(self, string_editor):
        self.string_list_layout.removeWidget(string_editor)
        self.string_editor_list.remove(string_editor)
        string_editor.close()
        string_editor.deleteLater()

    def add_string_editor(self, key=E):
        editor = StringEditor(self)
        editor.key_chooser.key = key
        self.string_list_layout.addWidget(editor)
        self.string_editor_list.append(editor)
        editor.show()

    def clear_string_editor_list(self):
        for editor in list(self.string_editor_list):
            self.remove_string_editor(editor)

